

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <time.h>

#include "transfer.h"

/*
    The procedures  signal_block   &   msgDefineMessage  are  called from the libxfer library  
      For this example code they can be noopos
*/

void signal_block ()  {}

int msgDefineMessage (u_int id, u_int xclass, u_int level, char *source, char *body) {

//  printf("id=%d, class=%d, level=%d, source=%s: %s\n",id, xclass, level, source, body);
  return 0;
}



void Usage(char *progname) {
    printf("Usage\n%s -n [server] -p [port] -f [file_name] -d [format] -t [ticks] -s [buffersize] -v [v]\n",progname);
    printf("\tserver is the IP address or name of server (default localhost)\n");
    printf("\tport is the base TCP port to use (default 10305)\n");
    printf("\tfile_name defines a file to be read for data rather than using internally generated data (default no file - use internal data)\n");
    printf("\tformat defines the data to be generated (default 0)\n");
    printf("\tticks is the delay between sending data blocks in millisecs (default 0)\n");
    printf("\tbuffersize is data block size in Kbytes (default 64)\n");
    printf("\tv is the setting for the verbose flag (default 0)\n");
    exit(1);
}
        
int main(int argc, char **argv) {

   int ID = 0;

   int  Sync;
   int Ctime;

    char Buffer[(64*1024)+32];
    int * BufferL;
    short * BufferS;
    char *TS_Server= (char*)"localhost";
    int Port=10305;
    int InFlag =0;     /* no file input by default  */
    int Format = 0;
    FILE *InStream;
    char *FileName= (char*)"f0.dat";
    char fname[32];
    unsigned int stream = 1;
    int DelayTicks=0;     /* millisecs  */
    int NumBlocks=1;
    int verbose=0;
    int i, j;
    int BufferSize=64*1024;
    int num;
    int b;
    int rc;
    int Fblocks;

    struct timeval Time;
    int InitialTime;
    int diff;
    long StartTimeL, StartTimeS;
    long EndTimeL, EndTimeS;
    long rate;

    long long usecs;
    long long bytes;

    struct timespec time_request;

    if (argc >1) {
        for(i=1;i <argc;i++) {
            if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
                switch(argv[i][1]) {
                    case 'f':
                        FileName = argv[++i];
                        break;
                    case 'i':
                        ID = atoi(argv[++i]);
                        break;
                    case 'd':
                        Format = atoi(argv[++i]);
                        break;
                    case 'n':
                        TS_Server = argv[++i];
                        break;
                    case 'p':
                        Port = atoi(argv[++i]);
                        break;
                    case 't':
                        DelayTicks = atoi(argv[++i]);
                        break;
                    case 's':
                        BufferSize = atoi(argv[++i])*1024;
                        break;
                    case 'N':
                        NumBlocks = atoi(argv[++i]);
                        break;
                    case 'v':
                        verbose = 1;
                        break;
                    default:
                        Usage(argv[0]);
                        break;
                }
            }
            else
                Usage(argv[0]);
        }
    }

//    check the file name

     sprintf(fname,"f%d.dat",ID);
     printf("%d: reading from %s\n",ID,fname);
     InStream = fopen(fname,"r");
     if (InStream == NULL) {
       printf ("%d: Unable to connect to data file\n",ID);
       exit(16);
     }

//    connect the TCP stream

    (void) transferSetVerbose(verbose);
    (void) transferUseOverlap(0);
    (void) transferNice(1);
    (void) transferMultiBlockSize(ID, BufferSize);
    (void) transferMultiPort(ID, Port);

    rc = transferMultiInit(ID, TS_Server);
    if (rc != 0) {
       printf ("%d: Unable to connect to server\n",ID);
       fclose(InStream);
       exit(16);
    }

    printf ("%d: Resetting communication with server %s\n",ID,TS_Server);
    
    gettimeofday(&Time, NULL);

    BufferL = (int *) &Buffer[0];
    BufferS = (short *) &Buffer[0];

    num = 0;
    Fblocks = 0;

    InitialTime = Time.tv_sec;
    printf("%d: Start at %d\n",ID, InitialTime);

    StartTimeL = Time.tv_sec;
    StartTimeS = Time.tv_usec;
    
    for (;;)
    {

    if (NumBlocks-- < 0) goto eof;

       if (DelayTicks) {

           time_request.tv_sec = DelayTicks/1000;
           time_request.tv_nsec = (DelayTicks - (time_request.tv_sec * 1000)) * 1000000;
           i = nanosleep(&time_request, NULL);
        }
         
       if (verbose) printf("%d: Transfer %d bytes\n",ID,BufferSize-32);

         Sync = Ctime;

          switch (Format) {
            default:
            case 0:

              rc = fread(&Buffer[8],1,BufferSize,InStream);  /*  note 32-24 = 8 */
              Fblocks++;
              if (rc == 0) {goto eof;}
              if (rc != BufferSize) {printf("%d: fread returns rc=%d\n",ID,rc);}

             break;
            case 1:

              rc = fread(&Buffer[32],1,BufferSize,InStream);
              Fblocks++;
              if (rc == 0) {goto eof;}
              if (rc != BufferSize) {printf("%d: fread returns rc=%d\n",ID,rc);}

             break;
         }

        if (verbose) printf("%d: 0x%x 0x%x 0x%x \n",ID,BufferL[7], BufferL[8], BufferL[9]);
         rc = transferMultiTxData (ID, &Buffer[32] , stream, BufferL[7]);
        if (verbose) printf("%d: Done with code=%d\n",ID,rc);

         num++;
         if ((num/100)*100 == num) {

             gettimeofday(&Time, NULL);

             diff = Time.tv_sec-InitialTime;

             EndTimeL = Time.tv_sec;
             EndTimeS = Time.tv_usec;

             usecs = ((EndTimeL-StartTimeL)*1000000) + (EndTimeS-StartTimeS);
             bytes = 1000*BufferSize;

             rate = ((1000000*bytes)/1024)/usecs;

             printf ("%d: sent %d blocks after %d; current rate = %d Kbytes/sec\n",ID, num, diff, rate);

             StartTimeS = EndTimeS;
             StartTimeL = EndTimeL;

        }
       
    }   //  end of for loop

eof: 

     printf ("%d: Finished at end of file\n",ID);

     fclose(InStream);
     exit(0);


}
