


#include <string.h>
#include <stdio.h>

#define N_DATALENGTH 16384    // in units of words  (ie 4096 => 16 Kbyte)

#include <stdlib.h> /* getenv, exit */
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>

void checkDATAbuffer(char*, int);
void printbufferL(char*, int);
void byteswop32(char*, int);
void byteswop316(char*, int);


FILE * InStream;
char *FileName= (char*)"R3B.dat";

unsigned int DataBuffer[N_DATALENGTH];
size_t count = N_DATALENGTH*4;


void Usage(char *progname) {
    fprintf(stderr,"Usage\n%s -f [file path name]\n",progname);
    fprintf(stderr,"\tfile path name is file name used to read data\n");
    exit(1);
}


int main(int argc, char *argv[], char *envp[])
{

   int i;
   ssize_t bytes;
   char fname[32];
   int blocks = 0;

//    command line arguments

    if (argc >1) {
        for(i=1;i <argc;i++) {
            if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
                switch(argv[i][1]) {
                    case 'f':
                        FileName = argv[++i];
                        break;
                    default:
                        Usage(argv[0]);
                        break;
                }
            }
            else
                Usage(argv[0]);
        }
    }

       sprintf(fname,"%s",FileName);
       InStream = fopen(fname,"r");

       if (InStream == NULL) {printf("read from %s failed\n",fname); exit(16);}

       printf("reading from %s\n",fname);

      for (;;) {
         bytes = fread((void*) &DataBuffer[0], 1, count, InStream);
         if (bytes != count) break;

         blocks++;

/*
    note that each 64K byte block starts with a 24 byte header which is constructed within the data receiver program 
*/

         printbufferL((char *)&DataBuffer[0], bytes);
         printf("data length = %d bytes\n", DataBuffer[5]);

//         byteswop32((char *)&DataBuffer[6], bytes-24);        /*    for TDR format   */

          checkDATAbuffer((char *)&DataBuffer[6], DataBuffer[5]);

      }

      printf ("reading data file terminated after %d blocks with bytes = %d\n", blocks, bytes);

      fclose(InStream);

   exit(0);

   return(0);
}

void checkDATAbuffer(char* p, int l) 
{

    int len;
    int module, hit, channel, ASIC;
    int traceflag;
    int trace_data ,trace_adc, trace_sync;
	
    unsigned int chan;
    unsigned int * eventbuffer;
    unsigned int data;
    unsigned short adc;
    unsigned short icode;
    unsigned int ifield;
    unsigned int timestamp;
    int i;
    int count;
    unsigned long long timestampf;
    unsigned long long WRTimeL=0, WRTimeH=0, WRTimeF=0;

        traceflag = 1;
	trace_data = 0 ;
	trace_sync = 1 ;
        trace_adc = 1;

    len = l/4;          
    eventbuffer = (unsigned int *)p;

    if (trace_data) {

                printf("buffer len=%d\n",len);

                printf(">");
                for (i = 0; i < 32; i++)
                {
                     printf("0x%08lx ", eventbuffer[i]);
                }
                printf("\n");
    }

    len = l/4;                 /*   len in units of int  */
    eventbuffer = (unsigned int *)p;
    if(traceflag) printf("Block received length %d\n", len );

    while (len >= 2)
    {

        data = *eventbuffer++;
        len--;
        timestamp = *eventbuffer++;
        len--;

        switch ((data >> 30 ) & 3) {

             case 3:             /*  ADC data word */

                hit = (data >> 29) & 1;
                module = (data >> 23) & 63; 
                ASIC = (data >> 19) & 15; 
                channel = (data >> 12) & 127;
                adc = (unsigned short) (data & 0x00000fff);
                timestampf = ((unsigned long long) WRTimeF & 0xfffffffff0000000) | (unsigned long long) (timestamp & 0x0fffffff);

    if (traceflag && trace_adc )  printf("adc data: 0x%04x hit=%d module=%d asic=%d channel=%d: 0x%08lx 0x%08lx 0x%016llx\n", adc, hit, module, ASIC, channel, data, timestamp, timestampf);
                
                break;
     
             case 2:    /*  information data word */

                icode = (data >> 20) & 0x0f;    /*  information code */
                ifield = data & 0x000fffff;     /*  information field */
                module = (data >> 24) & 0x3f;

                switch (icode) {
 
                    case 2:         /* PAUSE timestamp */
                       timestampf = ((unsigned long long) ifield << 28) | (unsigned long long) timestamp;
      if (traceflag && trace_sync )   printf("received PAUSE: m=%d, ts=0x%08lx 0x%08lx 0x%016llx\n", module, ifield, timestamp, timestampf);
                       break;

                    case 3:         /* RESUME timestamp */
                       timestampf = ((unsigned long long) ifield << 28) | (unsigned long long) timestamp;
      if (traceflag && trace_sync)   printf("received RESUME: m=%d, ts=0x%08lx 0x%08lx 0x%016llx\n", module, ifield, timestamp, timestampf);
                       break;

                    case 4:         /* White Rabbit timestamp L */
                       timestampf = ((unsigned long long) ifield << 28) | (unsigned long long) timestamp;
                       WRTimeL = timestampf;
      if (traceflag && trace_sync)   printf("%received WR timestamp L: m=%d, ts=0x%08lx 0x%08lx 0x%016llx\n", module, ifield, timestamp, timestampf);
                       break;

                    case 5:         /* White Rabbit timestamp H */
                       timestampf = ((unsigned long long) ifield << 28) | (unsigned long long) timestamp;
                       WRTimeH = timestampf;
                       WRTimeF = ((unsigned long long) ifield << 48) | WRTimeL;
      if (traceflag && trace_sync)   printf("%received WR timestamp H: m=%d, ts=0x%08lx 0x%08lx 0x%016llx 0x%016llx\n", module, ifield, timestamp, WRTimeH, WRTimeF);
                       break;

                    default:        /* undefined code */
                       printf("received undefined information code: %d\n", icode);
                       break;
                }
        
                break;

            case 1:       /* wave form data */
                printf("Wave form data is not supported!!!\n");
                len  = 0;
                break;

             case 0:       /*  bad format */
             default:    
                printf("bad event data (%d %d) - (0x%08lx 0x%08lx)\n", l/4, len, data, timestamp);
                len = 0;
                break;
         }
    } /* end of while */

}


void printbufferL(char* p, int l) 
{

    int len;
    int i, j;
    unsigned int * eventbuffer;

    len = l/4;                 /*   len in units of int  */
    eventbuffer = (unsigned  int *)p;

      printf("buffer len=%d bytes\n",l);
      printf(">\n");

  for (j=0;j<16;j++) {
    for (i=0;i<8;i++) {printf("0x%08lx ", *eventbuffer++);}
      printf("\n");
  }

}


void byteswop32(char* p, int l) 
{
    int len;
    char *ps;
    char *pd;
    char b0, b1, b2, b3;

   len = l/4;
   ps = p;
   pd = p;


   while (len > 0)
   {
       b0 = *ps++;
       b1 = *ps++;
       b2 = *ps++;
       b3 = *ps++;

       *pd++ = b3;
       *pd++ = b2;
       *pd++ = b1;
       *pd++ = b0;
   
       len--;
   }
}

void byteswop16(char* p, int l) 
{
    int len;
    char *ps;
    char *pd;
    char b0, b1;

   len = l/2;
   ps = p;
   pd = p;


   while (len > 0)
   {
       b0 = *ps++;
       b1 = *ps++;

       *pd++ = b1;
       *pd++ = b0;
   
       len--;
   }
}








