
#     this is loaded into the DataAcquisitionControl namespace

# variable started 0

if {![info exists started]} {variable started 0}

if {$started != 0} {

     variable JS ""
     variable INFO ""
     variable LogFlag

     if {$LogFlag == 1} {InsertLog "Last Updated: [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"}

    do_click
}

if {$started == 0} {           ;#  first time load code and define local variables
    set started 1

    variable JS ""
    variable INFO ""

    variable LogText ""
    variable LogFlag 0
    variable LogCount 0

    variable ShowLogFlag 0

    if {![info exists ::DataAcquisition::IncludeElectronics]} {set ::DataAcquisition::IncludeElectronics "enabled"}
    if {![info exists ::DataAcquisition::IncludeReadout]} {set ::DataAcquisition::IncludeReadout "enabled"}
    if {![info exists ::DataAcquisition::IncludeHistogramming]} {set ::DataAcquisition::IncludeHistogramming "enabled"}
    if {![info exists ::DataAcquisition::IncludeDataTransfer]} {set ::DataAcquisition::IncludeDataTransfer "enabled"}
    if {![info exists ::DataAcquisition::IncludeDataStorage]} {set ::DataAcquisition::IncludeDataStorage "enabled"}

    if {![info exists ::DataAcquisition::EnableHistogramming]} {set ::DataAcquisition::EnableHistogramming 1}
    if {![info exists ::DataAcquisition::EnableMerge]} {set ::DataAcquisition::EnableMerge 0}
    if {![info exists ::DataAcquisition::DataTransferStreams]} {set ::DataAcquisition::DataTransferStreams 1}
    if {![info exists ::DataAcquisition::EnableDataTransfer]} {set ::DataAcquisition::EnableDataTransfer 1}
    if {![info exists ::DataAcquisition::EnableDataTransfer2]} {set ::DataAcquisition::EnableDataTransfer2 0}
    if {![info exists ::DataAcquisition::EnableTape]} {set ::DataAcquisition::EnableTape 0}
    if {![info exists ::DataAcquisition::RDOGo]} {set ::DataAcquisition::RDOGo 0}

    variable allservers 0

#    code (int)  state (string)

#       0           stopped
#       1           going
#       2           reset
#       3           setting up
#       4           changing to going
#       5           changing to stopped
#       6           error   
#       7           changing to reset
#       8           undefined 
#       9           executing

    variable AcqCode 8
    variable AcqState "Unknown"
    variable AcqInfo "Unknown"

    foreach server $::DataAcquisition::ACQSERVERS {set ::DataAcquisition::AcqSysCode($server) 8}
    foreach server $::DataAcquisition::ACQSERVERS {set ::DataAcquisition::AcqSysState($server) "Unknown"}

    variable AcqParameter "None"
    variable AcqSetOption "None"
    variable AcqGetOption "Null"

    set NRows [expr ([llength $::DataAcquisition::ACQSERVERS] + 7)/8]
    set NCols 8

    variable SState
    for {set k 1} {$k <= [expr $NRows*$NCols]} {incr k}  {set SState($k) 0}
    for {set k 1} {$k <= [llength $::DataAcquisition::ACQSERVERS]} {incr k}  {set SState($k) 1}

   proc  show_variables {} {
       variable AcqCode 
       variable AcqState
       variable AcqInfo
        global DataAcquisitionControlServerHost
        global MIDAS_DataAcquisitionControlServer_Exists
       variable AcqParameter
       variable AcqSetOption
       variable AcqGetOption
       variable allservers
       variable SState

       InsertLog "allservers=$allservers"
       InsertLog "AcqServers=$::DataAcquisition::ACQSERVERS"
       InsertLog "AcqServer=$::DataAcquisition::ACQSERVER"

       set m {}
       for {set k 1} {$k <= [llength $::DataAcquisition::ACQSERVERS]} {incr k}  {lappend m "[set SState($k)]"}
       InsertLog "SState=$m"

       InsertLog "AcqCode=$AcqCode; AcqState=$AcqState; AcqInfo=$AcqInfo"
       InsertLog "AcqParameter=$AcqParameter; AcqSetOption=$AcqSetOption; AcqGetOption=$AcqGetOption"
       InsertLog "HistEnable=$::DataAcquisition::EnableHistogramming"
       InsertLog "Xfer streams=$::DataAcquisition::DataTransferStreams; Enables=$::DataAcquisition::EnableDataTransfer,$::DataAcquisition::EnableDataTransfer2"

       InsertLog "methods:[info procs DataAcquisitionControlClient__*]"

       if {[info exists MIDAS_DataAcquisitionControlServer_Exists]} then {
         InsertLog "MIDAS_DataAcquisitionControlServer_Exists = $MIDAS_DataAcquisitionControlServer_Exists"
       }

       if {[info exists DataAcquisitionControlServerHost]} then {
         InsertLog "DataAcquisitionControlServerHost = $DataAcquisitionControlServerHost"
       }

       do_getstate
       InsertLog "AcqCode=$AcqCode; AcqState=$AcqState; AcqInfo=$AcqInfo"
   }

    proc do_reset {} {
        global errorCode errorInfo
        global DataAcquisitionControlServerHost
        global MIDAS_DataAcquisitionControlServer_Exists
        variable AcqParameter
        variable allservers
        variable SState

        variable PreReset_Exists
        if {[info exists PreReset_Exists]} then {
           set rc [catch {do_PreReset} m]
           if {$rc != 0} {
              InsertLog "PreReset returned with an error\n$m"
           } else {
#              InsertLog "PreReset returned $m ([format "0x%x" $m])"
#              InsertLog "PreReset returned $m"
           }
        }

      set BAK $::DataAcquisition::ACQSERVER

      set k 0
      foreach acqserver $::DataAcquisition::ACQSERVERS {
          incr k
          if {$allservers == 0 && $acqserver != $::DataAcquisition::ACQSERVER} continue
          if {$SState($k) == 0} continue

          set ::DataAcquisition::ACQSERVER $acqserver

       if {[info exists MIDAS_DataAcquisitionControlServer_Exists] && $MIDAS_DataAcquisitionControlServer_Exists == 1} then {
           set rc [catch {interp eval "" ::namespace inscope ::urn:DataAcquisitionControlServer do_DoReset "$AcqParameter"} m]
       } else {
           if {$DataAcquisitionControlServerHost != $::DataAcquisition::ACQSERVER} {
                set DataAcquisitionControlServerHost  $::DataAcquisition::ACQSERVER
                DataAcquisitionControlClient_Configure
           }
           set rc [catch {DataAcquisitionControlClient__DoReset "$AcqParameter"} m]
       }

        if {$rc != 0} {
            InsertLog "RESET for $acqserver returned with an error\n$m\n$errorCode\n$errorInfo"
        } else {
#            InsertLog "RESET returned $m ([format "0x%x" $m])"
#            InsertLog "RESET returned $m"
        }

      }

      set ::DataAcquisition::ACQSERVER $BAK

        variable PostReset_Exists
        if {[info exists PostReset_Exists]} then {
           set rc [catch {do_PostReset} m]
           if {$rc != 0} {
              InsertLog "PostReset returned with an error\n$m"
           } else {
#              InsertLog "PostReset returned $m ([format "0x%x" $m])"
#              InsertLog "PostReset returned $m"
           }
        }

    }

    proc do_setup {} {
        global errorCode errorInfo
        global DataAcquisitionControlServerHost
        global MIDAS_DataAcquisitionControlServer_Exists
        variable AcqParameter
        variable allservers
        variable SState

        variable PreSetup_Exists
        if {[info exists PreSetup_Exists]} then {
           set rc [catch {do_PreSetup} m]
           if {$rc != 0} {
              InsertLog "PreSetup returned with an error\n$m"
           } else {
#              InsertLog "PreSetup returned $m ([format "0x%x" $m])"
#              InsertLog "PreSetup returned $m"
           }
        }

      set BAK $::DataAcquisition::ACQSERVER

      set k 0
      foreach acqserver $::DataAcquisition::ACQSERVERS {
          incr k
          if {$allservers == 0 && $acqserver != $::DataAcquisition::ACQSERVER} continue
          if {$SState($k) == 0} continue

          set ::DataAcquisition::ACQSERVER $acqserver

       if {[info exists MIDAS_DataAcquisitionControlServer_Exists] && $MIDAS_DataAcquisitionControlServer_Exists == 1} then {
           set rc [catch {interp eval "" ::namespace inscope ::urn:DataAcquisitionControlServer do_DoSetUp "$AcqParameter"} m]
       } else {
           if {$DataAcquisitionControlServerHost != $::DataAcquisition::ACQSERVER} {
                set DataAcquisitionControlServerHost  $::DataAcquisition::ACQSERVER
                DataAcquisitionControlClient_Configure
           }
           set rc [catch {DataAcquisitionControlClient__DoSetUp "$AcqParameter"} m]
       }

        if {$rc != 0} {

            InsertLog "SETUP for $acqserver returned with an error\n$m\n$errorCode\n$errorInfo"
        } else {
#            InsertLog "SETUP returned $m ([format "0x%x" $m])"
#            InsertLog "SETUP returned $m"
        }

      }

      set ::DataAcquisition::ACQSERVER $BAK

        variable PostSetup_Exists
        if {[info exists PostSetup_Exists]} then {
           set rc [catch {do_PostSetup} m]
           if {$rc != 0} {
              InsertLog "PostSetup returned with an error\n$m"
           } else {
#              InsertLog "PostSetup returned $m ([format "0x%x" $m])"
#              InsertLog "PostSetup returned $m"
           }
        }

    }

    proc do_stop {} {
        global errorCode errorInfo
        global DataAcquisitionControlServerHost
        global MIDAS_DataAcquisitionControlServer_Exists
        variable AcqParameter
        variable allservers
        variable SState


      set BAK $::DataAcquisition::ACQSERVER

      set k 0
      foreach acqserver $::DataAcquisition::ACQSERVERS {
          incr k
          if {$allservers == 0 && $acqserver != $::DataAcquisition::ACQSERVER} continue
          if {$SState($k) == 0} continue

          set ::DataAcquisition::ACQSERVER $acqserver

       if {[info exists MIDAS_DataAcquisitionControlServer_Exists] && $MIDAS_DataAcquisitionControlServer_Exists == 1} then {
           set rc [catch {interp eval "" ::namespace inscope ::urn:DataAcquisitionControlServer do_DoStop "$AcqParameter"} m]
       } else {
           if {$DataAcquisitionControlServerHost != $::DataAcquisition::ACQSERVER} {
                set DataAcquisitionControlServerHost  $::DataAcquisition::ACQSERVER
                DataAcquisitionControlClient_Configure
           }
           set rc [catch {DataAcquisitionControlClient__DoStop "$AcqParameter"} m]
       }

        if {$rc != 0} {
            InsertLog "STOP for $acqserver returned with an error\n$m\n$errorCode\n$errorInfo"
        } else {
#            InsertLog "STOP returned $m ([format "0x%x" $m])"
#           InsertLog "STOP returned $m"
        }

      }

      set ::DataAcquisition::ACQSERVER $BAK

    }

    proc do_go {} {
        global errorCode errorInfo
        global DataAcquisitionControlServerHost
        global MIDAS_DataAcquisitionControlServer_Exists
        variable AcqParameter
        variable allservers
        variable SState

      set BAK $::DataAcquisition::ACQSERVER

      set k 0
      foreach acqserver $::DataAcquisition::ACQSERVERS {
          incr k
          if {$allservers == 0 && $acqserver != $::DataAcquisition::ACQSERVER} continue
          if {$SState($k) == 0} continue

          set ::DataAcquisition::ACQSERVER $acqserver

       if {[info exists MIDAS_DataAcquisitionControlServer_Exists] && $MIDAS_DataAcquisitionControlServer_Exists == 1} then {
           set rc [catch {interp eval "" ::namespace inscope ::urn:DataAcquisitionControlServer do_DoGo "$AcqParameter"} m]
       } else {
           if {$DataAcquisitionControlServerHost != $::DataAcquisition::ACQSERVER} {
                set DataAcquisitionControlServerHost  $::DataAcquisition::ACQSERVER
                DataAcquisitionControlClient_Configure
           }
           set rc [catch {DataAcquisitionControlClient__DoGo "$AcqParameter"} m]
       }

        if {$rc != 0} {
            InsertLog "GO for $acqserver returned with an error\n$m\n$errorCode\n$errorInfo"
        } else {
#            InsertLog "GO returned $m ([format "0x%x" $m])"
#            InsertLog "GO returned $m"
        }

      }

      set ::DataAcquisition::ACQSERVER $BAK

    }

    proc do_setopt {option} {
        global errorCode errorInfo
        global DataAcquisitionControlServerHost
        global MIDAS_DataAcquisitionControlServer_Exists
        variable AcqSetOption
        variable allservers
        variable SState

     set AcqSetOption $option

      set BAK $::DataAcquisition::ACQSERVER
      set response ""

      set k 0
      foreach acqserver $::DataAcquisition::ACQSERVERS {
          incr k
          if {$allservers == 0 && $acqserver != $::DataAcquisition::ACQSERVER} continue
          if {$SState($k) == 0} continue

          set ::DataAcquisition::ACQSERVER $acqserver

       if {[info exists MIDAS_DataAcquisitionControlServer_Exists] && $MIDAS_DataAcquisitionControlServer_Exists == 1} then {
           set rc [catch {interp eval "" ::namespace inscope ::urn:DataAcquisitionControlServer do_DoOption "$AcqSetOption"} m]
       } else {
           if {$DataAcquisitionControlServerHost != $::DataAcquisition::ACQSERVER} {
                set DataAcquisitionControlServerHost  $::DataAcquisition::ACQSERVER
                DataAcquisitionControlClient_Configure
           }
           set rc [catch {DataAcquisitionControlClient__DoOption "$AcqSetOption"} m]
       }

        if {$rc != 0} {
            InsertLog "DO OPTION for $acqserver returned with an error\n$m\n$errorCode\n$errorInfo"
        } else {
#            InsertLog "DO OPTION returned $m ([format "0x%x" $m])"
#            InsertLog "DO OPTION returned $m"
        }

        append response "$m "

      }

      set ::DataAcquisition::ACQSERVER $BAK

     return "$response"

    }

    proc do_getopt {option} {
        global errorCode errorInfo
        global DataAcquisitionControlServerHost
        global MIDAS_DataAcquisitionControlServer_Exists
        variable AcqGetOption
        variable allservers
        variable SState

      set AcqGetOption $option

      set BAK $::DataAcquisition::ACQSERVER
      set response ""

      set k 0
      foreach acqserver $::DataAcquisition::ACQSERVERS {
          incr k
          if {$allservers == 0 && $acqserver != $::DataAcquisition::ACQSERVER} continue
          if {$SState($k) == 0} continue

          set ::DataAcquisition::ACQSERVER $acqserver

       if {[info exists MIDAS_DataAcquisitionControlServer_Exists] && $MIDAS_DataAcquisitionControlServer_Exists == 1} then {
           set rc [catch {interp eval "" ::namespace inscope ::urn:DataAcquisitionControlServer do_GetOption "$AcqGetOption"} m]
       } else {
           if {$DataAcquisitionControlServerHost != $::DataAcquisition::ACQSERVER} {
                set DataAcquisitionControlServerHost  $::DataAcquisition::ACQSERVER
                DataAcquisitionControlClient_Configure
           }
           set rc [catch {DataAcquisitionControlClient__GetOption "$AcqGetOption"} m]
           if {$rc == 0} {set m [second $m]}
       }

        if {$rc != 0} {
            InsertLog "GET OPTION for $acqserver returned with an error\n$m\n$errorCode\n$errorInfo"
            set m "Null"
        } else {
#            InsertLog "GetOption with parameter $AcqGetOption returned $m"
        }

        append response "$m "

      }

      set ::DataAcquisition::ACQSERVER $BAK

        return "$response"
    }

    proc do_getstate {} {
        global errorCode errorInfo
        global DataAcquisitionControlServerHost
        global MIDAS_DataAcquisitionControlServer_Exists
        variable allservers
        variable SState
        variable AcqCode 
        variable AcqState
        variable AcqInfo 

      set BAK $::DataAcquisition::ACQSERVER

      foreach acqserver $::DataAcquisition::ACQSERVERS {set ::DataAcquisition::AcqSysCode($acqserver) 8}
      foreach acqserver $::DataAcquisition::ACQSERVERS {set ::DataAcquisition::AcqSysState($acqserver) "Unknown"}

      set k 0
      foreach acqserver $::DataAcquisition::ACQSERVERS {
          incr k
          if {$allservers == 0 && $acqserver != $::DataAcquisition::ACQSERVER} continue
          if {$SState($k) == 0} continue

          set ::DataAcquisition::ACQSERVER $acqserver

          if {[info exists MIDAS_DataAcquisitionControlServer_Exists] && $MIDAS_DataAcquisitionControlServer_Exists == 1} then {
              set rc [catch {interp eval "" ::namespace inscope ::urn:DataAcquisitionControlServer do_GetState} m]
          } else {
              if {$DataAcquisitionControlServerHost != $::DataAcquisition::ACQSERVER} {
                  set DataAcquisitionControlServerHost  $::DataAcquisition::ACQSERVER
                  DataAcquisitionControlClient_Configure
              }
              set rc [catch {DataAcquisitionControlClient__GetState} m]
          }

          if {$rc != 0} {
             InsertLog "STATE for $acqserver returned with an error\n$m\n$errorCode\n$errorInfo"
             set AcqCode 6
             set AcqState "error"
             set AcqInfo "unknown"
          } else {
             set AcqCode [lindex $m 1]
             set AcqState [lindex $m 2]
             set AcqInfo [lindex $m 3]
#            InsertLog "STATE returned $acqserver: $AcqState"
          }
          set ::DataAcquisition::AcqSysCode($acqserver) $AcqCode
          set ::DataAcquisition::AcqSysState($acqserver) $AcqState
      }
      set ::DataAcquisition::ACQSERVER $BAK

      if {$allservers == 0} {return $AcqCode} 

#    monitoring multiple systems
#    logic to set a return code when not all are the same 

      set i 0
      foreach acqserver $::DataAcquisition::ACQSERVERS {
          if {$i == 0} {
              set AcqCode $::DataAcquisition::AcqSysCode($acqserver)
              incr i
          } else {
              if {$::DataAcquisition::AcqSysCode($acqserver) != $AcqCode} {
                 set AcqCode 8; set AcqState "Complex"; set AcqInfo "complex state"
#                 set ::DataAcquisition::AcqSysCode($acqserver) 8
#                 set ::DataAcquisition::AcqSysState($acqserver) "Undefined"
                 return $AcqCode
              }
          }
      }

      return $AcqCode
   }


   proc system {args} {

         set cgidata [ncgi::nvlist]

         set w [first [lindex $cgidata 1]]
         set cmd [second [lindex $cgidata 1]]

         switch -glob $cmd {

             do_stop  -
             do_go    -
             do_reset -
             do_setup {$cmd}

             DataAcqStatus      {do_DataAcqReport}
             RawData            {Information "[do_getopt RawData]"}
             RawGData           {Information "[do_getopt RawGData]"}

             SoftwareLoad       {do_setopt "SoftwareReload"}

             ResetElectronics   {do_setopt "ResetElectronics"}
             StopElectronics    {do_setopt "StopElectronics"}
             GoElectronics      {do_setopt "GoElectronics"}
             SetupElectronics   {do_setopt "SetupElectronics"}
             SaveHistograms     {do_setopt "HistogramSave"}
             ZeroHistograms     {do_setopt "HistogramZero"}

             TerminateExec      {do_setopt "TerminateExec"}
             LoadExec           {do_setopt "LoadExec"}
             FlashPgm           {do_FlashPgm}

             DisConnect0        {do_setopt "DisConnect=0"}
             DisConnect1        {do_setopt "DisConnect=1"}
             ReConnect0         {do_setopt "ReConnect=0"}
             ReConnect1         {do_setopt "ReConnect=1"}

             default   {InsertLog "unexpected system command:  $cmd"}
         }
   }
   proc do_RDOGo {} {

 	if {[do_getstate] == 0 } {InsertLog "You should not Start Readout unless in GO state" ; return }

        set r [do_setopt "RDOGo_Operate"]
   }


   proc do_FlashPgm {} {
        global env
        global errorCode errorInfo
        global DataAcquisitionControlServerHost
        global MIDAS_DataAcquisitionControlServer_Exists
        variable AcqSetOption
        variable allservers
        variable SState

	set local_time [ exec date ]
	InsertLog "started at $local_time"

        set r [do_setopt "FlashPgm"]

      set response ""

      set k 0

      foreach acqserver $::DataAcquisition::ACQSERVERS {
          	if {$allservers == 0 && $acqserver != $::DataAcquisition::ACQSERVER} continue

             switch -glob $acqserver {
                 nnr3b*  {set file_name [file join  /Embedded XilinxLinux ppc_4xx rfs $acqserver tmp FlashPgm.log ]; set fsize 180}
                 nnaida* {set file_name [file join  $env(MIDASBASE) XilinxLinux ppc_4xx rfs $acqserver tmp FlashPgm.log ]; set fsize 120}
                 default {return}
             }

		for { set i 0 } { $i < 100 } { incr i } {
			after 1000 ; # sleep for a second
			set file_size [ file size $file_name ]
			if { $file_size > $fsize } break ; # removed -v setting of command flashcp
		}
         	set z [catch {open $file_name r} file_id]
		InsertLog "File => $file_name , $z , $file_id, $file_size, $i"
		set contents [ read $file_id ]
		InsertLog "%%%%%%%%\n$contents%%%%%%%%"
		set ok_test [ string last finished $contents ]
 		set fail_test [ string last FAILED $contents ] 
InsertLog "********* $ok_test   $fail_test"
		if { $ok_test > 0  & $fail_test < 0 } { # if an error is reported by the console then FAILED is added to the log
			InsertLog " $acqserver Programmed ok.($ok_test)"
			set k [ expr $k + 1 ] 
		} else { 
			InsertLog " $acqserver Programming FAILED.($ok_test): Try directly using telnet"
			set error_message [ string range $contents [ expr $fail_test + 6 ] end ]
			InsertLog "$error_message"
			InsertLog "$contents"
		}
         	set z [catch {close $file_id } close_error]
	}
        InsertLog "FlashPgm returns: $k FEE64s programmed"

   }

     proc do_DataAcqReport {} {
        variable allservers

      set BAK $::DataAcquisition::ACQSERVER

      foreach server $::DataAcquisition::ACQSERVERS {
          if {$allservers == 0 && $server != $::DataAcquisition::ACQSERVER} continue

          set ::DataAcquisition::ACQSERVER $server

          InsertLog "server = $server"
          set host [do_getopt "NetVar=NetVar.TS_Server"]
          set port [do_getopt "NetVar=NetVar.TS_Port"]
          set var  [do_getopt "NetVar=NetVar.DACQ.Ctxenable"]
          InsertLog "Data Stream #0- $host:$port enable=$var"
          set host [do_getopt "NetVar=NetVar.TS2_Server"]
          set port [do_getopt "NetVar=NetVar.TS2_Port"]
          set var  [do_getopt "NetVar=NetVar.DACQ.Ctx2enable"]
          InsertLog "Data Stream #1- $host:$port enable=$var"

      }

      set ::DataAcquisition::ACQSERVER $BAK

    }

    proc do_GO {} {do_go}
    proc do_STOP {} {do_stop}
    proc do_SETUP {} {do_setup}
    proc do_RESET {} {do_reset}
    proc do_getSTATE {} {do_getstate}

     proc do_click {} {
       variable cgidata
       variable started
       variable LogFlag
       variable ShowLogFlag

       variable allservers
       variable SState
       global MIDAS_DataAcquisitionCode

       set cgidata [ncgi::nvlist]

       if {$LogFlag == 1} {InsertLog "cgidata= ([llength $cgidata]) $cgidata"}

       if {[llength $cgidata] == 0} then {return}

       if {[llength $cgidata] < 2} then {
           InsertLog "CGI data is too short: $cgidata"
       } else {
           if {[lindex $cgidata 0] == "Query"} then {do_query; return}
           if {[lindex $cgidata 0] == "Action"} then {do_action; return}
           if {[lindex $cgidata 0] == "Set"} then {do_set; return}
           if {[lindex $cgidata 0] != "Widget"} then {
               InsertLog "Invalid CGI data: $cgidata"
           } else {

              set w [first [lindex $cgidata 1]]
              set vl [second [lindex $cgidata 1]]
              set p [third [lindex $cgidata 1]]

#    previously                 
#                   DOSTOP     {do_stop}
#                   DOGO       {do_go}
#    changed at Daresbury 23 June;  at RIKEN  5 May


               switch -glob $w {
                   RESET    {set started 0}
                   CLEAR    {EmptyLog}
                   ELOG     {PrintLog}
                   RELOAD   {}
                   VARIABLES {show_variables}

                   HIDE     {set ShowLogFlag 0}
                   SHOW     {set ShowLogFlag 1}
                   LOGON    {set LogFlag 1}
                   LOGOFF   {set LogFlag 0}

                   DORESET    {do_RESET}
                   DOSETUP    {do_SETUP}
                   DOSTOP     {do_STOP}
                   DOGO       {do_GO}

		   RDOGO      {do_RDOGo}

                   UPDATE    {}

                   SERVER     {set ::DataAcquisition::ACQSERVER $vl}
                   ALLSERVERS {set allservers [expr ($allservers + 1) & 1]}
                   SSTATE     {set SState($vl) [expr ([set SState($vl)] + 1) & 1]}

                   OPT      {
                       switch $vl {
                          1  {
                              switch $p {
                                  enable  {set ::DataAcquisition::IncludeElectronics "enabled"}
                                  disable {set ::DataAcquisition::IncludeElectronics "disabled"}
                              }
                          }
                          2  {
                              switch $p {
                                  enable  {set ::DataAcquisition::IncludeReadout "enabled"}
                                  disable {set ::DataAcquisition::IncludeReadout "disabled"}
                              }
                          }
                          3  {
                              switch $p {
                                  enable  {set ::DataAcquisition::IncludeHistogramming "enabled"}
                                  disable {set ::DataAcquisition::IncludeHistogramming "disabled"}
                              }
                          }
                          4  {
                              switch $p {
                                  enable  {set ::DataAcquisition::IncludeDataTransfer "enabled"}
                                  disable {set ::DataAcquisition::IncludeDataTransfer "disabled"}
                              }
                          }
                          5  {
                              switch $p {
                                  enable  {set ::DataAcquisition::IncludeDataStorage "enabled"}
                                  disable {set ::DataAcquisition::IncludeDataStorage "disabled"}
                              }
                          }
                       }
                   }

                   HIST     {
                       set BAK $allservers
                       switch $vl {
                           on  {set ::DataAcquisition::EnableHistogramming 1}
                           off {set ::DataAcquisition::EnableHistogramming 0}
                           allon  {set allservers 1; set ::DataAcquisition::EnableHistogramming 1}
                           alloff {set allservers 1; set ::DataAcquisition::EnableHistogramming 0}
                       }
                       do_setopt "HistEnable=$::DataAcquisition::EnableHistogramming"
                       set allservers $BAK
                   }

                   MERGE  {
                       InsertLog "set merge state to $vl"
                       switch $vl {
                           on  {set ::DataAcquisition::EnableMerge 1}
                           off {set ::DataAcquisition::EnableMerge 0}
                       }
                   }

                   XFER     {
                       if {[do_getstate] == 1} {InsertLog "You should not change this when GOING"}
                       set BAK $allservers
                       switch $vl {
                           on  {set ::DataAcquisition::EnableDataTransfer 1}
                           off {set ::DataAcquisition::EnableDataTransfer 0}
                           allon  {set allservers 1; set ::DataAcquisition::EnableDataTransfer 1}
                           alloff {set allservers 1; set ::DataAcquisition::EnableDataTransfer 0}
                       }
                       if {$::DataAcquisition::DataTransferStreams == 1} {
                             do_setopt "XferEnable=$::DataAcquisition::EnableDataTransfer"
                       }
                       if {$::DataAcquisition::DataTransferStreams == 2} {
                             do_setopt "Xfer2Enable=[set ::DataAcquisition::EnableDataTransfer]+[set ::DataAcquisition::EnableDataTransfer2]"
                       }
                       
                       set allservers $BAK
                   }

                   TAPE {
                       InsertLog "set tapeserver state to $vl"
                       switch $vl {
                           on  {set ::DataAcquisition::EnableTape 1}
                           off {set ::DataAcquisition::EnableTape 0}
                       }
                   }

                   XFER2     {
                       if {[do_getstate] == 1} {InsertLog "You should not change this when GOING"}
                       set BAK $allservers
                       switch $vl {
                           on  {set ::DataAcquisition::EnableDataTransfer2 1}
                           off {set ::DataAcquisition::EnableDataTransfer2 0}
                           allon  {set allservers 1; set ::DataAcquisition::EnableDataTransfer2 1}
                           alloff {set allservers 1; set ::DataAcquisition::EnableDataTransfer2 0}
                       }
                       do_setopt "Xfer2Enable=[set ::DataAcquisition::EnableDataTransfer]+[set ::DataAcquisition::EnableDataTransfer2]"

                       set allservers $BAK
                   }

                   SYSTEM   {system}

                   MainLog {}
                   undefined {}
                   default {InsertLog "unexpected CGI data:  $cgidata"}
               }
           }
       }

       do_getstate
       set ::DataAcquisition::EnableHistogramming [do_getopt "HistEnable"]
       if {$::DataAcquisition::DataTransferStreams == 1} {
           set ::DataAcquisition::EnableDataTransfer [do_getopt "XferEnable"]
       }
       if {$::DataAcquisition::DataTransferStreams == 2} {
           set ::DataAcquisition::EnableDataTransfer ""
           set ::DataAcquisition::EnableDataTransfer2 ""
           foreach item [do_getopt "Xfer2Enable"] {
              lappend ::DataAcquisition::EnableDataTransfer [first [split $item +]]
              lappend ::DataAcquisition::EnableDataTransfer2 [second [split $item +]]
           }
       }
    }    ;#  end of do_click


   proc do_action {} {
      global Httpd
      upvar #0 Httpd[set Httpd(currentSocket)] data

     set sock $Httpd(currentSocket)

     set m ""

     append m "do_action\n"
     append m "uri = $data(uri)\n"

     set info [second [split $data(uri) ?]]

     if {[string compare -length 7 "Action=" "$info"] == 0} {

         set action [string range $info 7 end]

         append m "action = $action\n"

         switch $action {
            do_RESET   {do_RESET}
            do_SETUP   {do_SETUP}
            do_STOP    {do_STOP}
            do_GO      {do_GO}
	    do_RDOGo   {do_RDOGo}

            undefined -
            default {append m "unexpected action: $action\n"}
         }
     }

      append m "*****action request done*****\n"

      Httpd_ReturnData $sock "http" $m 

   }

   proc do_query {} {
      global Httpd
      upvar #0 Httpd[set Httpd(currentSocket)] data
      variable AcqCode
      variable AcqState
      variable AcqInfo

     set sock $Httpd(currentSocket)

     set m ""

     append m "do_query\n"
     append m "uri = $data(uri)\n"

     set info [second [split $data(uri) ?]]

     if {[string compare -length 6 "Query=" "$info"] == 0} {

         set query [string range $info 6 end]

         append m "query = $query\n"

         set qlist [split $query &]

         foreach q $qlist {

            switch $q {
              do_getSTATE   {append m "Server state = [do_getSTATE]\nAcqCode = $AcqCode; AcqState = $AcqState; AcqInfo = $AcqInfo\n"}

              undefined -
              default {append m "unexpected query: $q\n"}
            }
         }
      }

      append m "*****query request done*****\n"

      Httpd_ReturnData $sock "http" $m 

   }

   proc do_set {} {
      global Httpd
      upvar #0 Httpd[set Httpd(currentSocket)] data

     set sock $Httpd(currentSocket)

     set m ""

     append m "do_set\n"
     append m "uri = $data(uri)\n"

     set info [second [split $data(uri) ?]]

     if {[string compare -length 4 "Set=" "$info"] == 0} {

         set parm [string range $info 4 end]

         append m "set = $parm\n"

         set plist [split $parm &]

         foreach item $plist {
            set p [first [split $item =]]
            set v [second [split $item =]]

            switch $p {
              undefined -
              default {append m "unexpected set: $p\n"}
            }
         }
      }

      append m "*****parameter set request done*****\n"

      Httpd_ReturnData $sock "http" $m 

   }

}   ;# end of if {$started == 0}




