
#    This is a component of Spectrum.tcl

if {$Started == 0} {
    
    variable backpeakoptions
    variable peakfindoptions
    variable brumfitoptions

      set backpeakoptions(backtype) flatline
      set backpeakoptions(peaktype) gauss
      set backpeakoptions(weight)   count
      set backpeakoptions(shape)    differen
      set backpeakoptions(peaks)    1
      set backpeakoptions(order)    1
      set backpeakoptions(shapesfile) "no"

      set peakfindoptions(low-chan) 0
      set peakfindoptions(high-chan) 8191
      set peakfindoptions(fwhm)     20
      set peakfindoptions(accept)   10
      
      set brumfitoptions(shape) gauss
      set brumfitoptions(widthopt) different
      set brumfitoptions(peakopt) different
      set brumfitoptions(order) 2
      set brumfitoptions(widths) 5.0
      set brumfitoptions(skews) 10.0

}
    
    proc SpectrumReference {} {
       variable currentgallery
       variable currentoverlap
        upvar #1 dbSpecPathName($currentgallery,$currentoverlap) SpecPathName
        return $SpecPathName
    }

   proc do_view {args} {
       variable currentgallery
       variable currentoverlap
       
       upvar #1 dbXMin($currentgallery) XMin
       upvar #1 dbXMax($currentgallery) XMax
       upvar #1 dbYMin($currentgallery) YMin
       upvar #1 dbYMax($currentgallery) YMax

       variable SpecLimit
       variable WindowSize
       variable QuikLoad

       variable wxscale
       variable wyscale

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           "expand"        {do_expand}
            zoomin -
           "zoom in"       {do_zoom In}
            zoomout -
           "zoom out"      {do_zoom Out}
            times2         {do_YMax [expr $YMax*2]}
            divide2        {do_YMax [expr $YMax/2]}
            left           {do_XMin [expr $XMin - ($WindowSize/$wxscale)]; do_XMax [expr $XMax - ($WindowSize/$wxscale)]}
            right          {do_XMin [expr $XMin + ($WindowSize/$wxscale)]; do_XMax [expr $XMax + ($WindowSize/$wxscale)]}
            up             {do_YMin [expr $YMin + ($SpecLimit/$wyscale)]; do_YMax [expr $YMax + ($SpecLimit/$wyscale)]}
            down           {do_YMin [expr $YMin - ($SpecLimit/$wyscale)]; do_YMax [expr $YMax - ($SpecLimit/$wyscale)]}
            reset          {do_reset}
            invert         {variable ColorScheme; set ColorScheme [expr ($ColorScheme+1)&1]; do_ColorScheme}
           "skip channel 0"    {do_skip "true"}
           "unskip channel 0"  {do_skip "false"}
           "enable QuikLoad"   {set QuikLoad 1}
           "disable QuikLoad"  {set QuikLoad 0}
            default         {InsertLog "Unknown view function $f"}
       }
   }
   
   proc do_arrange {args} {
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable currentgallery
       variable currentoverlap

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           "remove all spectra"   {
                 RemoveAllSpectra
           }
           "remove active spectrum"   {
                 set g $currentgallery
                 for {set o 1} {$o <= $numoverlap} {incr o} {
                     if {$dbInUse($g,$o) == 1} {set dbInUse($g,$o) 0}
                 }
                 for {set j 1} {$j <= $MaxTags} {incr j} {
                     if {$dbtagx($g,$j) == 0} {continue}
                     set dbtagx($g,$j) 0; set dbtagy($g,$j) 0
                 }
                 for {set g 1} {$g <= $numgallery} {incr g} {
                     if {$dbInUse($g,1) == 1} {set currentgallery $g; set currentoverlap 1; break}
                 }
           }
           "save layout"   {
                 do_saveLayout
           }
           "restore layout"   {
                 do_restoreLayout
           }

           default                {InsertLog "Unknown arrange function $f"}
       }
   }
   
   proc do_analysis {args} {
       variable QuikLoad
       variable MemSas

       if {$MemSas == 1 && $QuikLoad == 1} {
           InsertLog "analysis functions are not available for MemSas histograms when QuikLoad is enabled"
           return
       }

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           integrate            {do_integrate}
          "peak to total"      {do_peaktototal}
          "max channel counts"  {do_maxcounts}
          "max channel counts (range)" {do_rangemaxcounts}
          "total counts"        {do_totalcounts}
          "total counts (range)"       {do_rangetotalcounts}
          "peak fit"            {do_peakfit}
          "peak fit options"    {}
          "peak find"           {do_peakfind}
          "peak find options"   {}
          "buf fit"            {do_buffit}
          "buf fit options"    {}
           default              {InsertLog "Unknown analysis function $f"}
       }
   }
   
   proc do_tagsfits {args} {

       set f [lreplace [first $args] 0 0]
   
       switch $f {
          "clear tags"    {do_cleartags}
          "clear fits"    {do_clearfits}
           default        {InsertLog "Unknown tagsfits function $f"}
       }
   }
   
   proc do_actions1 {args} {
       variable showit
       variable MemSas
       variable DiscSas
       variable DirName
       variable currentgallery
       variable currentoverlap
       
       upvar #1 dbSpecDim($currentgallery,$currentoverlap) SpecDim
       
       set f [lreplace [first $args] 0 0]
   
       switch $f {
           details   {do_details 0}
           title     {do_title 0}
           create    {
               if {$SpecDim == 1} {do_create1D}
               if {$SpecDim == 2} {do_create2D}
               set showit 0
           }
           delete     {do_delete; set showit 0}
           zero       {do_zero; set showit 0}
           inquire    {do_inquiry [SpectrumName]}
           inquireall {
              if {$MemSas == 1}  {do_inquiry *}
              if {$DiscSas == 1} {do_inquiry [file join $DirName *]}
           }
           reset   {do_reset}
           refresh {}
           default {InsertLog "Unknown analysis function $f"}
       }
   }
   
    proc do_actions2 {args} {
    
       set f [second [first $args]]
       set vl [third [first $args]]

       switch $f {
           tag     {do_pushtag $vl}
           default {InsertLog "Unknown analysis function $args ($f)  ($vl)"}       
       }
    }

    proc do_saveLayout {} {
        global env
        global tcl_platform
        variable MaxGalleries 
        variable MaxOverlaps 
        variable numgallery
        variable layout

       variable dbInUse
       variable dbServerName

#    check for and if necessary create a temporary directory

#       if {[midas-check-for-temp] == 0} {return -1}
#       set path [file join $env(MIDASTempDir) LayOut1.mlf]

       set path [file join /tmp LayOut[set layout].mlf]  ;  InsertLog "Using $path"

       set z [catch {file delete -force $path}]   ;#  in case already exists

       set z1 [catch {open $path w} m1]

       if {$tcl_platform(platform) != "windows"} {
           set z2 [catch {file attributes $path -permissions rwxrwxrwx} m2]
       } else {set z2 0}

       if {$z1 != 0} then {
          InsertLog "Cannot create $path - $m1"
       }
       if {$z2 != 0} then {
          InsertLog "error accessing $path - $m2"
       }

       set f $m1

       puts $f "$numgallery"

       for {set g 1} {$g<=$MaxGalleries} {incr g} {
          if {$dbInUse($g,1) == 0} {continue}
          for {set o 1} {$o <= $MaxOverlaps} {incr o} {
             if {$dbInUse($g,$o) == 0}  {continue}
             puts $f "gallery=$g overlap=$o server=$dbServerName($g,$o) spectrum=[SpectrumName $g $o]"
          }
       }

       close $f

       InsertLog "Current gallery layout saved"
    }

    proc do_restoreLayout {} {
        global env
        variable MemSas
        variable DiscSas
        variable DirName
       variable currentgallery
       variable currentoverlap
       variable numgallery
        variable layout
 
       RemoveAllSpectra

#       if {[midas-check-for-temp] == 0} {return -1}
#       set path [file join $env(MIDASTempDir) LayOut1.mlf]

       set path [file join /tmp LayOut[set layout].mlf]  ;  InsertLog "Using $path"

       set z1 [catch {open $path r} f]

       if {$z1 != 0} then {
          InsertLog "error accessing $path - $f"
       }

       set lines [split [read $f] \n]

       close $f

       set BAK $::DataAcquisition::ACQSERVER

       set i 0
       foreach item $lines {
            if {$item == ""} break
            if {$i == 0} {incr i; set numgallery $item; continue}
#  gallery=1 overlap=1 server=vplx2.home spectrum=/MIDAS/SpecData/CAEN/v1751psd#1.QL0.msf

        foreach element [split $item " "] {

           switch [first [split $element =]] {
             gallery {
               set currentgallery [second [split $element =]]
             }
             overlap {
               set currentoverlap [second [split $element =]]
             }
             server {
               set server [second [split $element =]]
             }
             spectrum {
               set spectrum [second [split $element =]]
               set DirName [file dirname $spectrum]
             }
             default {}
           }
        }

             InsertLog "restore server=$server spectrum=$spectrum to gallery=$currentgallery overlap=$currentoverlap"
	     set ::DataAcquisition::ACQSERVER $server
             do_configure    ;# configure for SpectrumServerHost - must come after setting ACQSERVER
             SelectSpectrum [file tail $spectrum]

       }

       if {$MemSas == 1}  {do_inquiry *}
       if {$DiscSas == 1} {do_inquiry [file join $DirName *]}

       set ::DataAcquisition::ACQSERVER $BAK
       do_configure

       InsertLog "Saved gallery layout restored"
    }
    
    proc do_peakfit {} {
       variable backpeakoptions
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
       variable dbfit
       variable MaxFits
       variable nfits
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
      if {$ntags < 6} {InsertLog "6 limits are required"; return}
      
      set tags {}
      for {set i 1} {$i <= 6} {incr i} {
          lappend tags $dbtagx($currentgallery,[expr $ntags-6+$i])
      }
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set use_shapesfile ""
                if {$backpeakoptions(shapesfile) == "yes"} {set use_shapesfile "-shapesfile $shapesfile"}
                
                set execute "exec backpeak  \
                  -spectrum $dbSpecPathName($g,$o) \
                  -bgtype $backpeakoptions(backtype) \
                  -peaktype $backpeakoptions(peaktype) \
                  -weighting $backpeakoptions(weight) \
                  -shape $backpeakoptions(shape) \
                  -order $backpeakoptions(order) \
                  -peaks $backpeakoptions(peaks) \
                  -centroid compute \
                  $use_shapesfile \
                  -limit $tags"

                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "[last $m]\n"
                } else {
                   InsertLog "Peakfit returns an error: $dbSpecPathName($g,$o): $m"
                   InsertLog "\nUsage:  6 tags are required  \
                      \nfirst 2 define the peak to be fitted\
                      \nremaining 4 tags define 2 background areas which must be on different sides of the peak"
                   return
                }
                
                if {$nfits == $MaxFits} {
                    InsertLog "All fits in use"
                } else {
      
                    incr nfits
      
                    set fchan  [first [first [second $m]]]
                    set fnum  [second [first [second $m]]]
                    set fvals [second [second $m]]
      
                    set dbfit($g,$nfits) ""
                    for {set i 0} {$i < $fnum} {incr i} {
                       append dbfit($g,$nfits) "\[$fchan,[lindex $fvals $i]\],"
                       incr fchan
                    }
                }
                
             }
          }
       }
    }
    
    proc do_buffit {} {
       variable brumfitoptions
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
      if {$ntags < 3} {InsertLog "at least 3 tags are required"; return}
      
      set tags {}
      for {set i 1} {$i <= $ntags} {incr i} {
          lappend tags $dbtagx($currentgallery,$i)
      }
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec brumfit  \
                  -spectrum $dbSpecPathName($g,$o) \
                  -shape $brumfitoptions(shape) \
                  -widthopt $brumfitoptions(widthopt) \
                  -peakopt $brumfitoptions(peakopt) \
                  -order $brumfitoptions(order) \
                  -limit [first $tags] [second $tags] \
                  -peaks [tail [tail $tags]] \
                  -widths $brumfitoptions(widths) \
                  -skews $brumfitoptions(skews)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "$dbSpecPathName($g,$o)[first $m]\n"
                } else {
                   InsertLog "Brumfit returns an error: $dbSpecPathName($g,$o): $m"
                   InsertLog "\nUsage:  2 + n tags are required  \
                      \nfirst 2 define the range to be considered\
                      \nremaining tags define the peaks to be fitted"
                }
             }
          }
       }
    }
    
    proc  do_peakfind {} {
       variable currentgallery
       variable currentoverlap
       variable MaxTags
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       variable peakfindoptions
    
       upvar #1 dbXMin($currentgallery) XMin
       upvar #1 dbXMax($currentgallery) XMax
       
       set peakfindoptions(low-chan) $XMin
       set peakfindoptions(high-chan) $XMax
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
                
                set execute "exec peakfind \
                   -spectrum $dbSpecPathName($g,$o) \
                   -limit $peakfindoptions(low-chan) $peakfindoptions(high-chan) \
                   -fwhm $peakfindoptions(fwhm) \
                   -accept $peakfindoptions(accept)"

                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "$execute => $m\n"
                } else {
                   InsertLog "PeakFind returns an error: $m"
                   return
                }

#    put a tag on each peak
 
                set i 0
                foreach item [split $m \n] {
                   incr i
                   if {$i < 3} {continue}

                   upvar #1 dbSpecRawData($g,1) SpecRawData   ;# take counts from overlap 1

                   set x [first [split [second $item] "("]]
               
                   set done 0
                   for {set j 1} {$j <= $MaxTags} {incr j} {
                      if {$dbtagx($g,$j) != 0} {continue}
                   
                      set dbtagx($g,$j) [expr round($x)]
                      set dbtagy($g,$j) [lindex $SpecRawData $dbtagx($g,$j)]
                   
                      incr done
                      break    ;#  done 
                   }
                   if {$done == 0} {InsertLog "too many tags for gallery $g. max supported = $MaxTags"}

                }
             }
          }
       }
    }
    
    proc do_integrate {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
       if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec integrate -limit  $dbtagx($currentgallery,$ntags) $dbtagx($currentgallery,[expr $ntags-1]) -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "$m\n"
                } else {
                   InsertLog "Integrate returns an error: $m"
                }
             }
          }
       }
    }
    
    proc do_peaktototal {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
       if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec peak-to-total -spectrum $dbSpecPathName($g,$o) -limit  $dbtagx($currentgallery,$ntags) $dbtagx($currentgallery,[expr $ntags-1])"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "$m\n"
                } else {
                   InsertLog "PeakToTotal returns an error: $m"
                }
             }
          }
       }
    }
   
    proc do_totalcounts {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable dbSpecPathName
       variable mode2
       
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec total-counts -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "Total-Counts for spectrum $m"
                } else {
                   InsertLog "Total-Counts for spectrum $dbSpecPathName($g,$o)"
                   InsertLog "Total-Counts returns an error: $m"
                }
             }
          }
       }
    }
    
    proc do_rangetotalcounts {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec total-counts  $dbtagx($currentgallery,$ntags) $dbtagx($currentgallery,[expr $ntags-1]) -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "Total-Counts from  $dbtagx($currentgallery,$ntags) to $dbtagx($currentgallery,[expr $ntags-1]) for spectrum $m"
                } else {
                   InsertLog "Total-Countsfrom  $dbtagx($currentgallery,$ntags) to $dbtagx($currentgallery,[expr $ntags-1]) for spectrum $dbSpecPathName($g,$o)"
                   InsertLog "Total-Counts returns an error: $m"
                }
             }
          }
       }
    }
   
    proc do_maxcounts {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable dbSpecPathName
       variable mode2
       
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec max-counts -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "Max-Counts for spectrum $m"
                } else {
                   InsertLog "Max-Counts for spectrum $dbSpecPathName($g,$o)"
                   InsertLog "Max-Counts returns an error: $m"
                }
             }
          }
       }
    }
    
    proc do_rangemaxcounts {} {
       variable MaxTags
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2

      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
      
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec max-counts  $dbtagx($currentgallery,$ntags) $dbtagx($currentgallery,[expr $ntags-1]) -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "Max-Counts from  $dbtagx($currentgallery,$ntags) to $dbtagx($currentgallery,[expr $ntags-1]) for spectrum $m"
                } else {
                   InsertLog "Max-Countsfrom  $dbtagx($currentgallery,$ntags) to $dbtagx($currentgallery,[expr $ntags-1]) for spectrum $dbSpecPathName($g,$o)"
                   InsertLog "Max-Counts returns an error: $m"
                }
             }
          }
       }
    }
    
   proc do_cont {def} {
       variable DefCont
       if {$def == "true" || $def == 1} {
         set DefCont 1
       } else {
         set DefCont 0
       }
   }

   proc do_skip {skip} {
       variable currentgallery
       variable currentoverlap
       
       upvar #1 dbXMin($currentgallery) XMin
       upvar #1 dbXMax($currentgallery) XMax
       
       variable Skip0
       
       if {$skip == "true" || $skip == 1} {
         set Skip0 1
       } else {
         set Skip0 0
         if {$XMin <= 2} {set XMin $SpecBase0}
       }
   }   
    
    proc do_zoom {w} {
       variable currentgallery
       variable numgallery
       variable MaxTags
       variable dbtagx
       variable dbXMin
       variable dbXMax
       variable dbSpecBase0
       variable dbSpecBase1
       variable dbSpecRange0
       variable dbSpecRange1
       variable dbSpecDim

       variable dbInUse
       variable mode2
       variable zxsize
       variable zysize
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
       if {$ntags < 1} {InsertLog "You need 1 tag for zoom"; return}

       for {set g 1} {$g <= $numgallery} {incr g} {
           if {$dbInUse($g,1) == 1} {
              if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
    
              set x $dbtagx($currentgallery,$ntags)
              set rangex [expr $dbXMax($g) - $dbXMin($g)]

puts "zoom  $x $rangex"
        
              if {$w == "In"} {
                set dbXMin($g) [expr $x - ($rangex/$zxsize)]
                set dbXMax($g) [expr $x + ($rangex/$zysize)]
              } else {
                set dbXMin($g) [expr $x - $rangex]
                set dbXMax($g) [expr $x + $rangex]      
              }

puts "zoom  $x $rangex $dbXMin($g) $dbXMax($g)"

              if {$dbSpecDim($g) == 2} {
                 variable dbYMin
                 variable dbYMax
                 variable dbtagy

                 set y $dbtagy($currentgallery,$ntags)
                 set rangey [expr $dbYMax($g) - $dbYMin($g)]

puts "zoom  $y $rangey"

                 if {$w == "In"} {
                   set dbYMin($g) [expr $y - ($rangey/$zxsize)]
                   set dbYMax($g) [expr $y + ($rangey/$zysize)]
                 } else {
                   set dbYMin($g) [expr $y - $rangey]
                   set dbYMax($g) [expr $y + $rangey]        
                 }

puts "zoom  $y $rangey  $dbYMin($g) $dbYMax($g)"

              }

              if {$dbXMin($g) < $dbSpecBase0($g,1)} {set $dbXMin($g)  $dbSpecBase0($g,1)}
              if {$dbXMax($g) > $dbSpecRange0($g,1)} {set $dbXMax($g)  $dbSpecRange0($g,1)}
              if {$dbSpecDim($g) == 2} {
                  if {$dbYMin($g) < $dbSpecBase1($g,1)} {set $dbYMin($g)  $dbSpecRange1($g,1)}
                  if {$dbYMax($g) > $dbSpecRange1($g,1)} {set $dbYMax($g)  $dbSpecRange1($g,1)}
              }

puts "zoom  $x $y $rangex $rangey $dbXMin($g) $dbXMax($g) $dbYMin($g) $dbYMax($g)"

           }
        }
    }
   
   proc do_pushtag {x} {
       variable currentgallery
       variable numgallery
       variable MaxTags
       
       variable dbInUse
       variable mode2
       variable dbtagx
       variable dbtagy
       
       for {set g 1} {$g <= $numgallery} {incr g} {
           if {$dbInUse($g,1) == 1} {
               if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
               
               upvar #1 dbSpecRawData($g,1) SpecRawData   ;# take counts from overlap 1
               
               set done 0
               for {set j 1} {$j <= $MaxTags} {incr j} {
                   if {$dbtagx($g,$j) != 0} {continue}
                   
                   set dbtagx($g,$j) [expr round($x)]
                   set dbtagy($g,$j) [lindex $SpecRawData $dbtagx($g,$j)]
                   
                   incr done
                   break    ;#  done 
               }
               if {$done == 0} {InsertLog "too many tags for gallery $g. max supported = $MaxTags"}
           }
       }
   }

   proc do_push2dtag {x y} {
       variable currentgallery
       variable numgallery
       variable MaxTags
       
       variable dbInUse
       variable mode2
       variable dbtagx
       variable dbtagy
       variable dbtagz

puts "2dtag $x $y"
       
       for {set g 1} {$g <= $numgallery} {incr g} {
           if {$dbInUse($g,1) == 1} {
               if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
               
               upvar #1 dbSpecData($g,1) SpecData   ;# take counts from overlap 1
               upvar #1 dbSpecRange0($g,1) SpecRange0
               upvar #1 dbSpecRange1($g,1) SpecRange1

               if {$x > $SpecRange0 || $y > $SpecRange1} return
               if {$x < 5 || $y < 5} return
               
               set done 0
               for {set j 1} {$j <= $MaxTags} {incr j} {
                   if {$dbtagx($g,$j) != 0} {continue}
                   
                   set dbtagx($g,$j) [expr round($x)]
                   set dbtagy($g,$j) [expr round($y)]

                   set i [expr ($dbtagx($g,$j) * $SpecRange0) + $dbtagy($g,$j)]
                   set dbtagz($g,$j) [lindex $SpecData $i]

                   incr done
                   break    ;#  done 
               }
               if {$done == 0} {InsertLog "too many tags for gallery $g. max supported = $MaxTags"}
           }
       }
   }


   proc do_reset {} {
       variable currentgallery
       variable numgallery
       variable dbInUse
       variable mode2
       
       variable WindowSize
       variable WindowSizeX
       variable WindowSizeY

       variable SpecLimit
       variable SelectionMode 
       
       for {set g 1} {$g<=$numgallery} {incr g} {
         if {$dbInUse($g,1) == 1} {
           if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
           
       upvar #1 dbXMin($g) XMin
       upvar #1 dbXMax($g) XMax
       upvar #1 dbYMin($g) YMin
       upvar #1 dbYMax($g) YMax
       
       upvar #1 dbSpecBase0($g,1) SpecBase0
       upvar #1 dbSpecRange0($g,1) SpecRange0
       upvar #1 dbSpecBase1($g,1) SpecBase1
       upvar #1 dbSpecRange1($g,1) SpecRange1
           
           set XMin $SpecBase0
           set XMax $SpecRange0
           set YMin $SpecBase1
           set YMax $SpecRange1

           set WindowSize [expr $XMax - $XMin]

           set WindowSizeX [expr $XMax - $XMin]
           set WindowSizeY [expr $YMax - $YMin]
           
         }
       }
       
       set SpecLimit 0
       set SelectionMode 0
   }
   
   proc do_expand {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable mode2
       variable MaxTags
       variable dbtagx
       variable dbtagy
       variable dbXMin
       variable dbXMax
       variable dbYMin
       variable dbYMax
       variable dbSpecDim

       variable WindowSize
       variable WindowSizeX
       variable WindowSizeY
   
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
       
       if {$ntags < 2} {InsertLog "You need 2 tags for expand"; return}
   
       for {set g 1} {$g<=$numgallery} {incr g} {
          if {$dbInUse($g,1) == 1} {
            if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current

            if {$dbSpecDim($currentgallery) == 1 || $dbSpecDim($currentgallery) == 2 } {
           
               if {$dbtagx($currentgallery,[expr $ntags-1]) < $dbtagx($currentgallery,$ntags)} {
                  set dbXMin($g) $dbtagx($currentgallery,[expr $ntags-1])
                  set dbXMax($g) $dbtagx($currentgallery,$ntags)
               } else {
                  set dbXMin($g) $dbtagx($currentgallery,$ntags)
                  set dbXMax($g) $dbtagx($currentgallery,[expr $ntags-1])
               }
       
               set WindowSize [expr $dbXMax($currentgallery) - $dbXMin($currentgallery)]
            }

            if {$dbSpecDim($currentgallery) == 2} {
           
               if {$dbtagy($currentgallery,[expr $ntags-1]) < $dbtagy($currentgallery,$ntags)} {
                  set dbYMin($g) $dbtagy($currentgallery,[expr $ntags-1])
                  set dbYMax($g) $dbtagy($currentgallery,$ntags)
               } else {
                  set dbYMin($g) $dbtagy($currentgallery,$ntags)
                  set dbYMax($g) $dbtagy($currentgallery,[expr $ntags-1])
               }
       
               set WindowSizeX [expr $dbXMax($currentgallery) - $dbXMin($currentgallery)]
               set WindowSizeY [expr $dbYMax($currentgallery) - $dbYMin($currentgallery)]
            }
       
          }
        }
       
        do_poptag
        do_poptag
   }

   proc do_Yexpand {ymin ymax} {
       variable currentgallery
       variable numgallery
       variable dbInUse
       variable dbYMin
       variable dbYMax
       variable mode2
       variable CustomYView

       for {set g 1} {$g<=$numgallery} {incr g} {
          if {$dbInUse($g,1) == 1} {
            if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current

            set dbYMin($g) $ymin
            set dbYMax($g) $ymax

          }
       }

       set CustomYView 1
  }
   
   proc do_cleartags {} {
       variable currentgallery
       variable numgallery
       variable dbInUse
       variable MaxTags
       variable mode2
       variable dbtagx
       variable dbtagy
       variable dbtagz
   
       for {set g 1} {$g<=$numgallery} {incr g} {
         if {$dbInUse($g,1) == 1} {
            if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
           
            for {set j 1} {$j <= $MaxTags} {incr j} {
                if {$dbtagx($g,$j) == 0} {continue}
                set dbtagx($g,$j) 0; set dbtagy($g,$j) 0; set dbtagz($g,$j) 0
            }
         }
       }
   }
   
   proc do_clearfits {} {
       variable currentgallery
       variable numgallery
       variable dbInUse
       variable MaxFits
       variable mode2
       variable dbfit
       
       for {set g 1} {$g<=$numgallery} {incr g} {
           if {$dbInUse($g,1) == 1} {
              if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
   
              for {set j 1} {$j <= $MaxFits} {incr j} {set dbfit($g,$j) ""}
           }
       }
   }
   
   proc do_poptag {} {
       variable MaxTags
       variable currentgallery
       variable dbtagx
       variable dbtagy
       variable dbtagz
        
       set ntags 0
       for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
       }
   
       if {$ntags < 1} {return}
       
       set dbtagx($currentgallery,$ntags) 0
       set dbtagy($currentgallery,$ntags) 0
       set dbtagz($currentgallery,$ntags) 0
   }

