

proc do_configure {} {
    global SpectrumServerHost

	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost  $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
}

proc do_create2D {} {
    global env
    variable SpecB
    variable SpecR
    variable SpecArgs
    global errorCode errorInfo
    
    global MIDAS_SpectrumAccessServer_Exists
    global SpectrumServerHost
    
    if {[info exists MIDAS_SpectrumAccessServer_Exists] && $MIDAS_SpectrumAccessServer_Exists == 1 && $SpectrumServerHost == "$env(ComputerName)"} then {
	set rc [catch {interp eval "" ::namespace inscope ::urn:SpectrumService  SpecNew  [SpectrumName] "$SpecB" "$SpecR" "$SpecArgs"} msg]
	set m [lindex $msg 5]
    } else {
	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost  $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
	set rc [catch {SpectrumClient__Create2DSpectrum [SpectrumName] "$SpecB" "$SpecR" "$SpecArgs"} msg]
	set m $msg
    }
    
    if {$rc == 0} {
	InsertLog "SpecNew returned: $m"
    } else {
	InsertLog "SpecNew returned with an error\n$msg\n$errorCode\n$errorInfo"
    }
}

proc do_create1D {} {
    global env
    variable SpecB
    variable SpecR
    global errorCode errorInfo
    
    global MIDAS_SpectrumAccessServer_Exists
    global SpectrumServerHost
    
    if {[info exists MIDAS_SpectrumAccessServer_Exists] && $MIDAS_SpectrumAccessServer_Exists == 1 && $SpectrumServerHost == "$env(ComputerName)"} then {
	set rc [catch {interp eval "" ::namespace inscope ::urn:SpectrumService  SpecNew  [SpectrumName] "$SpecB" "$SpecR"} msg]
	set m [lindex $msg 5]
    } else {
	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost  $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
	set rc [catch {SpectrumClient__Create1DSpectrum [SpectrumName] "$SpecB" "$SpecR"} msg]
	set m $msg
    }
    
    if {$rc == 0} {
	InsertLog "SpecNew returned: $m"
    } else {
	InsertLog "SpecNew returned with an error\n$msg\n$errorCode\n$errorInfo"
    }
}

proc do_inquiry {name} {
    global env
    global errorCode errorInfo
    variable SpectrumList
    
    global MIDAS_SpectrumAccessServer_Exists
    global SpectrumServerHost
    global SpectrumServerPort
    
    if {[info exists MIDAS_SpectrumAccessServer_Exists] && $MIDAS_SpectrumAccessServer_Exists == 1 && $SpectrumServerHost == "$env(ComputerName)"} then {
	set rc [catch {interp eval "" ::namespace inscope ::urn:SpectrumService SpecNames $name} msg]
	set m [lindex $msg 5]
    } else {
	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
	set rc [catch {SpectrumClient__InquireSpectra $name} msg]
	set m $msg
    }
    
    if {$rc == 0} {
#InsertLog "SpecNames returned: $msg"
	set SpectrumList {}
	foreach item $m {lappend SpectrumList $item}
	set SpectrumList [lsort -dictionary $SpectrumList]
    } else {
	InsertLog "SpecNames returned with an error\n$msg\n$errorCode\n$errorInfo"
    }
}

proc do_details {opt} {
    global env
    global errorCode errorInfo
    
    global MIDAS_SpectrumAccessServer_Exists
    global SpectrumServerHost
    
    if {[info exists MIDAS_SpectrumAccessServer_Exists] && $MIDAS_SpectrumAccessServer_Exists == 1 && $SpectrumServerHost == "$env(ComputerName)"} then {
	set rc [catch {interp eval "" ::namespace inscope ::urn:SpectrumService SpecDetails [SpectrumName]} msg]
	set m [lindex $msg 5]
    } else {
	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost  $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
	set rc [catch {SpectrumClient__SpectrumDetails [SpectrumName]} msg]
	set m $msg
    }
    
    if {$rc == 0} {
	if {$opt == 0} {
	    InsertLog "SpecDetails returned:"
	    foreach item $m {InsertLog "$item"}
	}
    } else {
	InsertLog "SpecDetails returned with an error\n$msg\n$errorCode\n$errorInfo"
    }
    
    return [list $rc $m]
}

proc do_title {opt} {
    global env
    global errorCode errorInfo
    variable DataList
    
    global MIDAS_SpectrumAccessServer_Exists
    global SpectrumServerHost
    
    if {[info exists MIDAS_SpectrumAccessServer_Exists] && $MIDAS_SpectrumAccessServer_Exists == 1 && $SpectrumServerHost == "$env(ComputerName)"} then {
	set rc [catch {interp eval "" ::namespace inscope ::urn:SpectrumService SpecGetTitle [SpectrumName]} msg]
	set m [lindex $msg 5]
    } else {
	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost  $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
	set rc [catch {SpectrumClient__SpectrumGetTitle [SpectrumName]} msg]
	set m $msg
    }
    
    if {$rc == 0} {
	if {$opt == 0} {
            InsertLog "SpecGetTitle returned: $m"
	}
	set DataList $m
    } else {
	InsertLog "SpecGetTitle returned with an error\n$msg\n$errorCode\n$errorInfo"
    }
    
    return [list $rc $m]
}

proc do_read1D {Name Base Range Compression} {
    global env
    global errorCode errorInfo
    
    global MIDAS_SpectrumAccessServer_Exists
    global SpectrumServerHost
    
    if {[info exists MIDAS_SpectrumAccessServer_Exists] && $MIDAS_SpectrumAccessServer_Exists == 1 && $SpectrumServerHost == "$env(ComputerName)"} then {
	set rc [catch {interp eval "" ::namespace inscope ::urn:SpectrumService SpecRead1DText $Name $Base $Range $Compression} msg]
	set m [lindex $msg 5]
    } else {
	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost  $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
	set rc [catch {SpectrumClient__Read1DSpectrumText $Name $Base $Range $Compression} msg]
	set m $msg
    }
    
    if {$rc == 0} {
	#           InsertLog "SpecRead1DText returned:"
	#           foreach item $m {InsertLog "$item"}
    } else {
	InsertLog "SpecRead1DText returned with an error\n$msg\n$errorCode\n$errorInfo"
    }
    
    return [list $rc $m]    
}


proc do_read2D {Name Base0 Range0 Base1 Range1 Compression0 Compression1} {
    global env
    global errorCode errorInfo
    
    global MIDAS_SpectrumAccessServer_Exists
    global SpectrumServerHost
    
    if {[info exists MIDAS_SpectrumAccessServer_Exists] && $MIDAS_SpectrumAccessServer_Exists == 1 && $SpectrumServerHost == "$env(ComputerName)"} then {
	set rc [catch {interp eval "" ::namespace inscope ::urn:SpectrumService SpecRead2DText $Name $Base0 $Range0 $Base1 $Range1 $Compression0 $Compression1} msg]
	set m [lindex $msg 5]
    } else {
	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost  $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
	set rc [catch {SpectrumClient__Read2DSpectrumText $Name $Base0 $Range0 $Base1 $Range1 $Compression0 $Compression1} msg]
	set m $msg
    }
    
    if {$rc == 0} {
	#           InsertLog "SpecRead2DText returned:"
	#           foreach item $m {InsertLog "$item"}
    } else {
	InsertLog "SpecRead2DText returned with an error\n$msg\n$errorCode\n$errorInfo"
    }
    
    return [list $rc $m]    
}


proc do_delete {} {
    global env
    global errorCode errorInfo
    
    global MIDAS_SpectrumAccessServer_Exists
    global SpectrumServerHost
    
    if {[info exists MIDAS_SpectrumAccessServer_Exists] && $MIDAS_SpectrumAccessServer_Exists == 1 && $SpectrumServerHost == "$env(ComputerName)"} then {
	set rc [catch {interp eval "" ::namespace inscope ::urn:SpectrumService SpecDelete [SpectrumName]} msg]
	set m [lindex $msg 5]
    } else {
	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost  $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
	set rc [catch {SpectrumClient__DeleteSpectrum [SpectrumName]} msg]
	set m $msg
    }
    
    if {$rc == 0} {
	InsertLog "SpecDelete returned: $m"
    } else {
	InsertLog "SpecDelete returned with an error\n$msg\n$errorCode\n$errorInfo"
    }
}

proc do_zero {} {
    global env
    global errorCode errorInfo
    
    global MIDAS_SpectrumAccessServer_Exists
    global SpectrumServerHost
    
    if {[info exists MIDAS_SpectrumAccessServer_Exists] && $MIDAS_SpectrumAccessServer_Exists == 1 && $SpectrumServerHost == "$env(ComputerName)"} then {
	set rc [catch {interp eval "" ::namespace inscope ::urn:SpectrumService SpecZero [SpectrumName]} msg]
	set m [lindex $msg 5]
    } else {
	if {$SpectrumServerHost != $::DataAcquisition::ACQSERVER} {
	    set SpectrumServerHost  $::DataAcquisition::ACQSERVER
	    SpectrumClient_Configure
	}
	set rc [catch {SpectrumClient__ZeroSpectrum [SpectrumName]} msg]
	set m $msg
    }
    
    if {$rc == 0} {
	InsertLog "SpecZero returned: $m"
    } else {
	InsertLog "SpecZero returned with an error\n$msg\n$errorCode\n$errorInfo"
    }
}

